<?php
/*
Plugin Name: Weather Widget by Calcatraz
Plugin URI: http://www.calcatraz.com/blog/free-wordpress-weather-widget-1995
Description: A simple, compact weather widget which displays current conditions and 3-day forecast for the selected location.
Author: Dan Mossop
Version: 1.2
Author URI: http://www.danmossop.com
*/


class calcatrazWeatherWidget extends WP_Widget {
	private $wid = 'calcatrazWeatherWidget'; // should be same as class name
	private $wname = "Weather Widget by Calcatraz";
	private $wdescr = 'A simple, compact weather widget which displays current conditions and 3-day forecast for selected location.';
	
	function calcatrazWeatherWidget() {
		$this->WP_Widget($this->wid, $this->wname, array('classname'=>$this->wid, 'description'=>$this->wdescr));
		wp_enqueue_script('jquery');
	}
 
	function form($instance) {
		$instance = wp_parse_args((array) $instance, array('title'=>'','city'=>''));
		// title
		$id = $this->get_field_id('title');
		$name = $this->get_field_name('title');
		$val = attribute_escape($instance['title']);
		echo <<<END
		<p><label for="$id">Title: <input class="widefat" id="$id" name="$name" type="text" value="$val" /></label></p> 
END;
		// city
		$cid = $this->get_field_id('city');
		$cname = $this->get_field_name('city');
		$cval = attribute_escape($instance['city']);
		echo <<<END
		<p><label for="$cid">City: <input class="widefat" id="$cid" name="$cname" type="text" value="$cval" /></label></p>
		<p style="font-size:smaller;font-style:italic">Must be one of: US Zipcode, UK Postcode, Canada Postalcode, IP address, city name, or Latitude/Longitude (as decimal)</p>	
END;
		// api key
		$kid = $this->get_field_id('key');
		$kname = $this->get_field_name('key');
		$kval = attribute_escape($instance['key']);
		echo <<<END
		<p><label for="$kid">API Key: <input class="widefat" id="$kid" name="$kname" type="text" value="$kval" /></label></p> 
		<p style="font-size:smaller;font-style:italic">This widget requires an API key from World Weather Online. You can get it free <a href="http://developer.worldweatheronline.com/">here</a></p>
END;
		// credits
		$crid = $this->get_field_id('credits');
		$crname = $this->get_field_name('credits');
		$checkbox = esc_attr($instance['credits']);
		$checked = checked('1', $checkbox, false);
		echo <<<END
		<p><label for="$crid">Show Credits: <input id="$crid" name="$crname" type="checkbox" value="1" $checked/></label></p> 
		<p style="font-size:smaller;font-style:italic">The World Weather Online free API requires a link crediting them to be shown. Their premium API does not.</p>
END;
	}
 
	function update($new_instance, $old_instance) {
		$instance = $old_instance;
		foreach(array('title', 'city', 'key', 'credits') as $k) {
			$instance[$k] = $new_instance[$k];
		}
		return $instance;
	}
 
	function widget($args, $instance) {
		extract($args, EXTR_SKIP);
		echo $before_widget;
		$title = empty($instance['title'])?' ':apply_filters('widget_title', $instance['title']);
		if (!empty($title)) { echo $before_title.$title.$after_title; }
    
		// WIDGET CODE GOES HERE
		$city = empty($instance['city'])?' ':apply_filters('widget_title', $instance['city']);
		$key = empty($instance['key'])?' ':apply_filters('widget_title', $instance['key']);
		$weatherdata = calcatrazweatherwidget_loadData($city, $key);
		calcatrazweatherwidget_showWidget($weatherdata, $instance['credits']);
		// END WIDGET CODE
		
		echo $after_widget;
	}
}

function calcatrazweatherwidget_loadData($city, $key) {
	$cityslug = md5($city);
	$file = dirname(__FILE__)."/data/$cityslug.json";
	if (!file_exists($file) or filemtime($file)<time()-60*60 or filesize($file)==0) { // refresh if > 1 hr old or empty
		$weatherdata = @file_get_contents('http://api.worldweatheronline.com/free/v1/weather.ashx?q='.urlencode(strtolower($city)).'&format=json&num_of_days=4&includelocation=yes&key='.urlencode(strtolower($key)));
		file_put_contents($file, $weatherdata);
	} else { 
		$weatherdata = file_get_contents($file);
	}
	return $weatherdata;
}

function calcatrazweatherwidget_showWidget($weatherdata, $showcredits) {
	$w = json_decode($weatherdata);
	if (isset($_SESSION['calcatrazweatherwidget_scale'])) { // user has selected preference
		$scale = ($_SESSION['calcatrazweatherwidget_scale']=='f')?'f':'c';
	} else { 
		$scale = (strpos($_SERVER['HTTP_ACCEPT_LANGUAGE'],'en-us')!==0)?'f':'c';
	}
	
	// current data
	$now = $w->data->current_condition[0];
	$now_icon = htmlentities($now->weatherIconUrl[0]->value);
	$now_temp_c = htmlentities($now->temp_C).'&deg;C';
	$now_temp_f = htmlentities($now->temp_F).'&deg;F';
	$now_desc = htmlentities($now->weatherDesc[0]->value);
	$now_temp = '<span class="temp" c="'.$now_temp_c.'" f="'.$now_temp_f.'">'.($scale=='c'?$now_temp_c:$now_temp_f).'</span>';
	
	// today forecast
	$today = $w->data->weather[0];
	$today_icon = htmlentities($today->weatherIconUrl[0]->value);
	$today_desc = htmlentities($today->weatherDesc[0]->value);
	$today_temp_max_c = htmlentities($today->tempMaxC).'&deg;C';
	$today_temp_min_c = htmlentities($today->tempMinC).'&deg;C';
	$today_temp_max_f = htmlentities($today->tempMaxF).'&deg;F';
	$today_temp_min_f = htmlentities($today->tempMinF).'&deg;F';
	
	$today_max = '<span class="temp" c="'.$today_temp_max_c.'" f="'.$today_temp_max_f.'">'.($scale=='c'?$today_temp_max_c:$today_temp_max_f).'</span>';
	$today_min = '<span class="temp" c="'.$today_temp_min_c.'" f="'.$today_temp_min_f.'">'.($scale=='c'?$today_temp_min_c:$today_temp_min_f).'</span>';
	
	// tomorrow forecast
	$tomorrow = $w->data->weather[1];
	$tomorrow_icon = htmlentities($tomorrow->weatherIconUrl[0]->value);
	$tomorrow_desc = htmlentities($tomorrow->weatherDesc[0]->value);
	$tomorrow_temp_max_c = htmlentities($tomorrow->tempMaxC).'&deg;C';
	$tomorrow_temp_min_c = htmlentities($tomorrow->tempMinC).'&deg;C';
	$tomorrow_temp_max_f = htmlentities($tomorrow->tempMaxF).'&deg;F';
	$tomorrow_temp_min_f = htmlentities($tomorrow->tempMinF).'&deg;F';
	$tomorrow_name = date("D",strtotime('tomorrow'));
	
	$tomorrow_max = '<span class="temp" c="'.$tomorrow_temp_max_c.'" f="'.$tomorrow_temp_max_f.'">'.($scale=='c'?$tomorrow_temp_max_c:$tomorrow_temp_max_f).'</span>';
	$tomorrow_min = '<span class="temp" c="'.$tomorrow_temp_min_c.'" f="'.$tomorrow_temp_min_f.'">'.($scale=='c'?$tomorrow_temp_min_c:$tomorrow_temp_min_f).'</span>';
	
	// day after forecast
	$dayafter = $w->data->weather[2];
	$dayafter_icon = htmlentities($dayafter->weatherIconUrl[0]->value);
	$dayafter_desc = htmlentities($dayafter->weatherDesc[0]->value);
	$dayafter_temp_max_c = htmlentities($dayafter->tempMaxC).'&deg;C';
	$dayafter_temp_min_c = htmlentities($dayafter->tempMinC).'&deg;C';
	$dayafter_temp_max_f = htmlentities($dayafter->tempMaxF).'&deg;F';
	$dayafter_temp_min_f = htmlentities($dayafter->tempMinF).'&deg;F';
	$dayafter_name = date("D",strtotime('tomorrow + 1 day'));
	
	
	$dayafter_max = '<span class="temp" c="'.$dayafter_temp_max_c.'" f="'.$dayafter_temp_max_f.'">'.($scale=='c'?$dayafter_temp_max_c:$dayafter_temp_max_f).'</span>';
	$dayafter_min = '<span class="temp" c="'.$dayafter_temp_min_c.'" f="'.$dayafter_temp_min_f.'">'.($scale=='c'?$dayafter_temp_min_c:$dayafter_temp_min_f).'</span>';
	
	// switch scale link
	$switch_scale_symbol = strtoupper($scale=='c'?'f':'c');
	$switch_scale_link =<<<END
		<a id="switchscale" href="javascript:;" onclick="
jQuery('#calcatraz-weather-widget span.temp').text(function(i){
	var elem=jQuery('#calcatraz-weather-widget span.temp').get(i);
	var temp = elem.getAttribute((jQuery('#switchscale').text()=='&deg;F')?'f':'c');
	return temp; });
	set_scale((jQuery('#switchscale').text()=='&deg;C')?'c':'f');
	jQuery('#switchscale').text((jQuery('#switchscale').text()=='&deg;F')?'&deg;C':'&deg;F');
" title="Switch between Celcius and Fahrenheit">&deg;$switch_scale_symbol</a>
END;
	
	// display widget html
	echo <<<END
</style>
<script>
function set_scale(scale) {
	jQuery(document).ready(function($){
            $.ajax({
                type: "GET",
                url: "index.php",
                data: { mywidget_request: "set_scale", mywidget_parameter1: scale},
                success : function(response) { }
            });
        });
}
</script>
<div id="calcatraz-weather-widget">
<div class="left">
  <img src="$now_icon"/>
  <div class="now"><p>$now_temp<br><span class="desc">$now_desc</span></p></div>
  <div class="switchscale">$switch_scale_link</div>
</div>
<div class="right">
<img src="$today_icon" title="$today_desc"/>
<span class="forecasttext row1">Today:</span>
<span class="forecasttext row1">$today_max / $today_min</span>
<img style="clear:both;margin-top:2px" src="$tomorrow_icon" title="$tomorrow_desc"/>
<span class="forecasttext">$tomorrow_name:</span>
<span class="forecasttext">$tomorrow_max / $tomorrow_min</span>
<img style="clear:both;margin-top:2px" src="$dayafter_icon" title="$dayafter_desc"/>
<span class="forecasttext">$dayafter_name:</span>
<span class="forecasttext">$dayafter_max / $dayafter_min</span>
</div>
</div>
END;
	if ($showcredits) { 
		echo<<<END
		<div id="calcatraz-weather-widget-credits">Data by <a href="http://www.worldweatheronline.com/" target="_blank">World Weather Online</a>. Widget by <a href="http://www.calcatraz.com/blog/free-wordpress-weather-widget-1995" title="Learn about the weather widget by Calcatraz">Calcatraz</a></div>
END;
	}  
}

// load css
wp_register_style('calcatrazweatherwidget-css', plugins_url('style.css',__FILE__ ));
wp_enqueue_style('calcatrazweatherwidget-css');

add_action( 'widgets_init', create_function('', 'return register_widget("calcatrazWeatherWidget");') );

// Function for handling AJAX requests
function calcatrazweatherwidget_request_handler() {
    // Check that all parameters have been passed
    if (isset($_GET['mywidget_request']) && isset($_GET['mywidget_parameter1'])) {
		if ($_GET['mywidget_request'] == 'set_scale') { 
			calcatrazweatherwidget_savescale(strip_tags($_GET['mywidget_parameter1'])); 
		}
        exit();
    } 
}
add_action('init', 'calcatrazweatherwidget_request_handler');

function calcatrazweatherwidget_savescale($val) { 
	$_SESSION['calcatrazweatherwidget_scale']=$val;
}

// enable sessions to store user preference
add_action('init', 'wwbc_startSession', 1);
add_action('wp_logout', 'wwbc_endSession');
add_action('wp_login', 'wwbc_endSession');
function wwbc_startSession() { if(!session_id()) { session_start(); } }
function wwbc_endSession() { session_destroy (); } 

?>